#ifndef AES_HPP
#define AES_HPP

#include <vector>
#include <cstdio>
#include <string>
#include <iostream>
using namespace std;

// Byte and Byte Array types
typedef unsigned char byte;
typedef std::vector<byte> byteArray;

// Word and Word Array types
typedef unsigned int word;
typedef std::vector<word> wordArray;

// Exceptions
class incorrectKeySizeException { };
class incorrectTextSizeException { };
class badStateArrayException { };

class AES
{
	public:
		AES (const byteArray& key);

		byteArray encrypt (const byteArray& plaintext) const;
		byteArray decrypt (const byteArray& ciphertext) const;

	private:
		unsigned int Nb;
		unsigned int Nk;
		unsigned int Nr;

		wordArray keySchedule;

		void KeyExpansion (const byteArray& key, wordArray& w) const;
		word SubWord (const word& input) const;
		word RotWord (const word& input) const;
		wordArray GetRoundKey (const int round) const;

		void SubBytes (byteArray& state) const;
		void ShiftRows (byteArray& state) const;
		void MixColumns (byteArray& state) const;
		void AddRoundKey (byteArray& state, const wordArray& w) const;

		void InvSubBytes (byteArray& state) const;
		void InvShiftRows (byteArray& state) const;
		void InvMixColumns (byteArray& state) const;

};

#endif /* AES_HPP */

/* vim: set ts=4 sts=4 sw=4 noet tw=112: */
