#include "aes.hpp"
#include <iostream>
#include <fstream>

static void print_by_16s (const byteArray b, std::string name)
{
	int i, j;

	std::cout << "Printing: " << name << "\n" << std::endl;

	for (i=0; i<b.size()/16; ++i)
	{
		for (j=0; j<16; ++j)
			std::printf ("%.2x ", b.at(i*16+j));

		std::printf ("\n");
	}
}


int main (int argc, char *argv[])
{

//#define DEBUG 1
//#define MY_TEST2 1
//#define PROBLEM2 1
//#define PROBLEM3 1

#if PROBLEM2
	int i;

	/* Read the Key */
	byteArray key;

	for (i=0; i<16; ++i)
		key.push_back (getchar());

	/* Create the AES object */
	AES aes (key);

	/* Read all characters of input */
	byteArray input;

	for (char c=getchar(); c != EOF; c=getchar())
	{
		input.push_back (c);

		if (input.size() == 16)
		{
			byteArray ciphertext = aes.encrypt (input);
			byteArray plaintext  = aes.decrypt (ciphertext);

			/* Print it */
			for (i=0; i<plaintext.size(); ++i)
				std::printf ("%c", plaintext.at(i));

			input.clear();
		}

	}
#endif // PROBLEM2

#if PROBLEM3
	std::ifstream fin ("Problem3.out", ios::binary);
	int i;

	byteArray key;

	for (i=0; i<24; ++i)
		key.push_back (fin.get());

	/* Create the AES Object */
	AES aes (key);

	byteArray input;

	/* Read the input and decrypt */
	for (char c=fin.get(); !fin.eof(); c=fin.get())
	{
		input.push_back (c);

		if (input.size() == 16)
		{
			byteArray plaintext = aes.decrypt (input);

			/* Print it */
			for (i=0; i<plaintext.size(); ++i)
				std::printf ("%c", plaintext.at(i));

		input.clear();
		}
	}
#endif //PROBLEM3

#if MY_TEST
	const int inputSize = 4096;

	byteArray key (16, 2);
	byteArray input (inputSize, 0);
	int i, j;

	for (i=0; i<inputSize; ++i)
		input.at (i) = i;

	AES aes (key);

	print_by_16s (input, "Input");

	byteArray ciphertext = aes.encrypt (input);

	print_by_16s (ciphertext, "Ciphertext");

	byteArray plaintext  = aes.decrypt (ciphertext);

	print_by_16s (plaintext, "Plaintext");
#endif

#if MY_TEST2
	int i;

	byteArray key (16, 0);

	for (i=0; i<16; ++i)
		key.at (i) = i;

	print_by_16s (key, "Key");

	byteArray plaintext (16, 0);

	for (i=0; i<16; ++i)
		plaintext.at (i) = (i << 4) | i;

	print_by_16s (plaintext, "Plaintext");

	AES aes (key);

	byteArray ciphertext = aes.encrypt (plaintext);

	print_by_16s (ciphertext, "Ciphertext");
#endif // MY_TEST2

	return 0;

}

/* vim: set ts=4 sts=4 sw=4 noet tw=120 nowrap: */
